/* Keep these headers */
#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>
#include <ti84pce.h>

/* Standard headers - it's recommended to leave them included */
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "screen8bpp.h"
#include "tools.h"

static const uint8_t* screen_buf = (uint8_t*)0xD40000;
static const uint8_t* offscreen_ptr = (uint8_t*)0xD52C00;    // vram "back buffer" (free to use when LCD is in 8bpp)  76.8k
uint8_t* brightness = (uint8_t*)0xF60024;
uint8_t minBrightness = 255;//229; // Do NOT set higher, may damage the hardware.
uint8_t maxBrightness = 49; // Do NOT set lower, may damage the hardware.
uint16_t* palette = (uint16_t*)0xE30200;
uint8_t bpp=16;

/* Your code here */

uint8_t* getOffscreen()
{
    return offscreen_ptr;   
}

uint8_t getBrightness()
{
    return *brightness;
}

void increaseBrightness(uint8_t targetBrightness)
{
    uint8_t i=0;
        // increase brightness
    for (i=*brightness; i>targetBrightness; i--) // yes this is the good direction
    {
        *brightness = i;
        nopwait(2000);
    }
    *brightness=targetBrightness;
}

void fillScreen(uint8_t color)
{
    memset(screen_buf, color, SCR_HEIGHT*SCR_WIDTH*bpp/8);
}

void fillLines(uint8_t color, uint8_t y1, uint8_t y2)
{
    uint8_t yi=min(y1,y2);
    uint8_t ys=max(y1,y2);
    memset(screen_buf+yi*SCR_WIDTH*bpp/8,color,(ys-yi+1)*SCR_WIDTH*bpp/8);
}

void initPalette()
{
    palette[0] = rgb2color(0, 0, 0);
    palette[1] = rgb2color(255, 0, 0);
    palette[2] = rgb2color(0, 255, 0);
    palette[3] = rgb2color(255, 0, 255);
    palette[4] = rgb2color(255, 255, 255);
    palette[5] = rgb2color(255, 171, 43);
    palette[6] = rgb2color(171, 255, 171);
    palette[7] = rgb2color(0, 0, 255);
    palette[8] = rgb2color(255, 171, 255);
    palette[9] = rgb2color(255, 255, 0);
}

void startScreen8bpp()
{
    // 320x240 8bpp
    fillScreen(0);
    *brightness=minBrightness;
    initPalette();
    (*(uint8_t*)0xE30018) = 0x27;
    bpp=8;
    fillScreen(0);
}

void endScreen8bpp()
{
    (*(uint8_t*)0xE30018) = 0x2D;
    bpp=16;
}

void gc_drawTile(uint8_t* data, uint16_t x, uint16_t y, uint16_t w, uint16_t h, uint8_t searchval, uint8_t replval)
{
    uint8_t* ptr_out = screen_buf;
    uint8_t* ptr_in  = data;
    uint8_t subst=(searchval!=replval)?1:0;
    uint8_t i;
    ptr_out += y*SCR_WIDTH + x;
    while (h>0)
    {
        memcpy(ptr_out, ptr_in, w);
        if(subst)
        {
            for(i=0;i<w;i++)
            {
                if(*ptr_out==searchval) *ptr_out=replval;
                ptr_out++;
            }
            ptr_out+=SCR_WIDTH-w;
        }
        else
            ptr_out += SCR_WIDTH;
        ptr_in  += w;
        h--;
    }
}